// 通过setHTML插入新内容时会触发这个事件
document.addEventListener("mindcontentinsert", function () {
    setTimeout(function () { // 延迟少量时间再执行相应的逻辑
        if (!RE.editor.isContentEditable) { // 在编辑器处于只读状态时执行一些初始化操作
            addImageClickListener() // 为所有图片添加点击事件
            adaptImageSize() // 调整<img>的宽度（最大为父容器的100%）
        } else { // 在编辑模式下对图像进行初始化
            initEditModeImgHolder()
        }
    }, 200);
});

// 监听图片插入事件
document.addEventListener("mindimageinsert", function (event) {
    setTimeout(function () { // 延迟少量时间再执行相应的逻辑
        var imgElement = document.getElementById(event.imageId)
        if (imgElement != null) {
            imgElement.addEventListener("click", function () {
                window.mindJSBridge.clickImage(RE.editor.isContentEditable, imgElement.src, imgElement.id);
            })
        }
    }, 200);
});

// 在编辑状态下为img添加修改图片大小的点击事件
function initEditModeImgHolder() {
    setTimeout(function () { // 延迟少量时间再执行相应的逻辑
        var images = document.getElementsByTagName('img');
        for (var i = 0; i < images.length; i++) {
            if (images[i].id == null || images[i].id.length == 0) {
                images[i].id = "mind_image_" + i
            }
            images[i].addEventListener("click", function () {
                window.mindJSBridge.clickImage(RE.editor.isContentEditable, this.src, this.id);
            })
        }
    }, 200);
}

// 发出图片已插入的事件
function dispatchImageInsertEvent(imageId) {
    var imageInsertEvent = document.createEvent("HTMLEvents");
    imageInsertEvent.initEvent("mindimageinsert", false, false);
    imageInsertEvent.imageId = imageId;
    document.dispatchEvent(imageInsertEvent)
}

// 发出内容已插入的事件（setHTML后触发）
function dispatchContentInsertEvent() {
    var contentInsertEvent = document.createEvent("HTMLEvents");
    contentInsertEvent.initEvent("mindcontentinsert", false, false);
    document.dispatchEvent(contentInsertEvent)
}

// 清除指定路径的css
function removeCss(href) {
    var cssTags = document.getElementsByTagName('link');

    for (var i = 0; i < cssTags.length; i++) {
        if (cssTags[i].type == "text/css" && cssTags[i].href == href) {
            cssTags[i].remove();
        }
    }
}

// 加载指定的JS文件（异步加载）
function loadJS(jsFilePath) {
    var body = document.body;
    var script = document.createElement("script");
    script.type = "text/javascript";
    script.src = jsFilePath;
    script.media = "all";
    body.appendChild(script);
}

/**
 * 手动执行script（在setHtml方法之后调用）
 * 主要用于文章详情页（比如加载复杂的scp文章）
 * setHtml设置的内容中包含<script>时需要手动调用这个方法
 */
function evalScript() {
    var contentDiv = document.getElementById('editor');
    var oldScript = document.getElementById('mindScript');
    if (contentDiv && oldScript) {
        contentDiv.removeChild(oldScript);
        var newScript = document.createElement('script');
        newScript.type = 'text/javascript';
        newScript.innerHTML = oldScript.innerHTML;
        contentDiv.appendChild(newScript);
    }
}

//为图片添加过渡加载效果
function initImage(obj) {
    obj.style.opacity = "0"
    var j = 0
    setInterval(function () {
        j += 0.2
        if (j <= 1) {
            obj.style.opacity = j
            return
        }
    }, 100)
}

// 为所有图片添加点击事件
function addImageClickListener() {
    var images = document.getElementsByTagName('img');
    for (var i = 0; i < images.length; i++) {
        images[i].onclick = function () {
            window.mindJSBridge.openImage(this.src);
        }
    }
}

// 调整<img>的宽度（最大为父容器的100%）
function adaptImageSize() {
    var images = document.getElementsByTagName('img');
    for (var i = 0; i < images.length; i++) {
        images[i].style.maxWidth = "100%"
    }
}

// 调整指定<img>的图片大小
function adaptTargetImageSize(imgId, widthSize) {
    var imageElement = document.getElementById(imgId)
    if (Number(widthSize) <= 0) {
        imageElement.style.width = "auto"
    } else {
        imageElement.style.width = widthSize + "%"
    }
    RE.callback() // 手动触发这个方法，保证编辑器里的的html内容是最新的
}

// 根据指定宽高添加图片
function insertImage(url, alt, width, height) {
    var imageCount = document.getElementsByTagName('img').length;
    var newImgId = "mind_image_" + imageCount;
    var imageHtml = "<img src='" + url + "' alt='" + alt + "' id='" + newImgId + "' width='" + width + "' height='" + height + "' +  />";
    RE.prepareInsert();
    RE.insertHTML(imageHtml);
    dispatchImageInsertEvent(newImgId)
}

/**
 * 插入图片
 * 宽/高自适应，宽度最大为编辑器的宽度
 */
function insertImageFillWidth(url, alt) {
    var imageCount = document.getElementsByTagName('img').length;
    var newImgId = "mind_image_" + imageCount;
    var imageHtml = "<img src='" + url + "' alt='" + alt + "' id='" + newImgId + "' style='max-width:100%;height:auto' />";
    RE.prepareInsert();
    RE.insertHTML(imageHtml)
    dispatchImageInsertEvent(newImgId)
}

/**
 * 插入音乐播放器（仅支持网易云音乐）
 */
function insertMusicPlayer(musicSrc) {
    var musicHtml = '<br><div class="mind-music-player"><iframe frameborder="no" border="0" marginwidth="0" marginheight="0" width="330" height="86" src="http:' + musicSrc + '" style="max-width: 100%;"></iframe></div><br>';
    RE.prepareInsert();
    RE.insertHTML(musicHtml)
}

/**
 * 插入iframe区域
 */
function insertFrameArea(iframeStr) {
    var iframeHtml = '<br><div class="mind-iframe-area">'+ iframeStr +'</div><br>';
    RE.prepareInsert();
    RE.insertHTML(iframeHtml)
}

/**
 * 获取编辑器中图片的数量（异步回调）
 * 通过JS桥将结果返回
 */
function getImageCount() {
    var images = document.getElementsByTagName('img');
    var imageCount = images.length;
    window.mindJSBridge.onImageCountGet(imageCount);
}

/**
 * 在当前位置插入内容（可以是普通文本或HTML）
 * prepareInsert是为了支持撤销功能
 */
function insertContent(content) {
    RE.prepareInsert();
    RE.insertHTML(content)
}

/**
 * 在当前位置插入纯文本内容
 * prepareInsert是为了支持撤销功能
 */
function insertPlainContent(content) {
    RE.prepareInsert();
    document.execCommand('insertText', false, content);
}

/**
 * 插入可折叠区域
 */
function insertFoldArea() {
    var foldAreaHtml = "<br><div class='collapsible-block'>   <div class='collapsible-block-folded'>   <a class='collapsible-block-link' href='javascript:;'>&#9654&nbsp;显示折叠内容&nbsp;</a>  </div>   <div class='collapsible-block-unfolded' style=''>    <div class='collapsible-block-unfolded-link'>    <a class='collapsible-block-link' href='javascript:;'>&#9660&nbsp;隐藏折叠内容&nbsp;</a>   </div>    <div class='collapsible-block-content'>  可折叠区域(修改为你想要的内容)  </div> </div> </div><br>"
    RE.prepareInsert();
    RE.insertHTML(foldAreaHtml)
}

// 插入可弹出提示窗口的按钮
function insertTipsButton(name, content) {
    RE.prepareInsert();
    RE.insertHTML('<input class="mind-button" value ="' + name + '" type="button" onclick="alert(\'' + content + '\')"/>')
}

// 插入打字机动画区域
function insertTypeAnimationArea(name, content, id) {
    RE.prepareInsert();
    RE.insertHTML('<style> #u-start-' + id + ', .textchange div.sitemember:nth-child(2) { display: none; }  .type-wrap-wrapper { position: relative; text-align: left; }  .type-wrapper { background: #555; }  .typearea { color: lime; font-family: &quot; Courier&quot; ; font-size: 1.2em; margin: 10px 0 0 10px; white-space: nowrap; overflow: hidden; width: 0; }  #u-start-' + id + ':target~.typearea { width: 100%; animation: type 4s steps(60, end); }  .typearea a { color: lime; text-decoration: none; }  .blink { animation: blink 1s infinite; }  @keyframes type { from { width: 0; } }  @keyframes blink { to { opacity: .0; } }  .buttonwrapper { text-align: center; }  .likebutton { padding: 0.3em 0.5em; border-radius:5px; text-decoration: none; font-size: 1em; background-color: #F4F4F4; border: 1px solid #AAA; color: #000; } </style>  <br> <div class="type-wrap-wrapper"> <div class="type-wrapper"> <div id="u-start-'+ id +'"></div> <div class="typearea"> <div class="textchange">  <div class="sitemember"> <p>' + content + '<span class="blink">|</span></p> </div> </div> </div> </div> <div class="buttonwrapper"> <p><a class="likebutton" href="#u-start-'+ id +'">' + name + '</a></p> </div> </div><br>')
}

// 插入笔记格式区域
function insertNoteArea() {
    RE.prepareInsert();
    RE.insertHTML('<br><div class="wiki-note"><div>&zwnj;</div></div><br>')
}

// 插入代码格式区域
function insertCodeArea() {
    RE.prepareInsert();
    RE.insertHTML('<br><div class="code"><div>&zwnj;</div></div><br>')
}

// 插入引用格式块
function insertOrSetBlockQuote() {
    // 如果当前存在选中内容，就直接设置引用格式，否则插入新的引用格式区域
    if (window.getSelection() && window.getSelection().toString()) {
        RE.setBlockquote();
    } else if (document.selection && document.selection.createRange().text) {
        RE.setBlockquote();
    } else {
        RE.prepareInsert();
        RE.insertHTML('<br><blockquote><div>&zwnj;</div></blockquote><br>')
    }
}

// 插入分割线
function insertHorizontalLine() {
    RE.prepareInsert();
    document.execCommand('InsertHorizontalRule', false, null)
}

// 插入App内使用的缩进（四个空格）
function insertMindIndent() {
    RE.prepareInsert();
    insertContent("&nbsp;&nbsp;&nbsp;&nbsp;")
}

// 插入表格
function insertTable(row, column, userHeader) {
    RE.prepareInsert();
    var rowNum = Number(row)
    var columnNum = Number(column)
    var isNeedInsertHeader = Boolean(userHeader)
    var finalHTML = '<table class="wiki-content-table">'

    if (isNeedInsertHeader) { // 是否插入表头
        finalHTML += '<tr>'
        for (var i = 0; i < columnNum; i++) {
            finalHTML += '<th style="min-width:40px;"></th>'
        }
        finalHTML += '</tr>'
    }

    for (var i = 0; i < rowNum; i++) {
        finalHTML += '<tr>'
        for (var j = 0; j < columnNum; j++) {
            finalHTML += '<td style="min-width:40px;"></td>'
        }
        finalHTML += '</tr>'
    }

    finalHTML += '</table>'
    RE.insertHTML(finalHTML)
}

// 选中全部内容
function selectAll() {
    document.execCommand("SelectAll", false, null)
}

// 删除全部内容
function clearAll() {
    document.execCommand("SelectAll", false, null)
    document.execCommand("Delete", false, null)
}


// 把光标移到末尾
function moveCursorToTail(obj) {
    if (window.getSelection) { // 解决浏览器的兼容问题，做如下条件判断
        obj.focus()
        var range = window.getSelection();
        range.selectAllChildren(obj);
        range.collapseToEnd(); // 光标移至最后
    }
    else if (document.selection) {
        var range = document.selection.createRange()
        range.moveToElementText(obj)
        range.collapse(false) // 光标移至最后
        range.select()
    }
}

/**
 * 插入新的编辑区域
 * 这样可以摆脱引用、可折叠区域等的影响
 */
function insertNewEditArea() {
    RE.editor.innerHTML += "<br>"
    moveCursorToTail(RE.editor)
}

// 设置是否仅仅允许粘贴纯文本
function setPasteOnlyPlain(onlyplain) {
    var pasteOnyPlain = Boolean(onlyplain)
    if (pasteOnyPlain) {
        insertNewEditArea() // 首先插入新的编辑区域，保证插入的文本不受旧格式的影响
        RE.editor.onpaste = function (event) {
            var e = event || window.event
            // 阻止默认粘贴
            e.preventDefault();
            // clipboardData的getData(fomat) 从剪贴板获取指定格式的数据
            var text = (e.originalEvent || e).clipboardData.getData('text/plain')
            document.execCommand("insertText", false, text);
        }
    } else {
        RE.editor.onpaste = function () {
            // do nothing
        }
    }

}

// 展开折叠区域
function doUnFoldAction(obj) {
    var unfoldedDiv = obj.parentNode; // 默认展示的区域
    if (unfoldedDiv.className !== "collapsible-block-folded") {
        unfoldedDiv = unfoldedDiv.parentNode; // 多找一层，避免异常情况
    }

    var foldRootDiv = unfoldedDiv.parentNode // 折叠区域根布局
    var foldedDiv = foldRootDiv.querySelector(".collapsible-block-unfolded")  // 默认隐藏的区域

    // 隐藏默认展示的div
    unfoldedDiv.style.display = "none"
    // 展示默认隐藏的div
    foldedDiv.style.display = ""
}

// 收起折叠区域
function doFoldAction(obj) {
    var foldedDiv = obj.parentNode.parentNode  // 默认隐藏的区域
    if (foldedDiv.className !== "collapsible-block-unfolded") {
        foldedDiv = foldedDiv.parentNode; // 多找一层，避免异常情况
    }

    var foldRootDiv = foldedDiv.parentNode // 折叠区域根布局
    var unfoldedDiv = foldRootDiv.querySelector(".collapsible-block-folded")  // 默认展示的区域

    // 展示默认展示的div
    unfoldedDiv.style.display = ""
    // 隐藏默认隐藏的div
    foldedDiv.style.display = "none"
}

// 插入密码锁区域
// 备忘：在插入的密码如果以0开头，就会被JS自动转为数字取消前置0，所以在点击验证按钮的时候，在传入的数字两侧增加了''单引号，相当于就是说明这里是字符串而不是数字
function insertCodeLockArea(password, numLength, id) {
    RE.prepareInsert();
    RE.insertHTML('<br><style type="text/css"> .codelock-info { display: none; }  .keypad { background: linear-gradient(to bottom, #D0D0D0 0%, #FFFFFF 80%, #CCCCCC 100%), #D4D4D4; padding: 6.4px; border-radius: 19.2px; margin: 25.6px; margin-left: auto; margin-right: auto; border: 2px solid #1F0D12; box-shadow: 0 3px 5px black, 0 3px 9px rgba(0, 0, 0, 0.5); border-collapse: separate; border-spacing: 6px; line-height: normal; }  .keypad-readout { background: linear-gradient(to top, #1A211E 1%, #000000 100%), black; color: #E22933; font-size: 22px; font-family: Consolas, monospace, sans-serif; font-weight: bold; border-radius: 3.84px; border-top-left-radius: 6.4px; border-top-right-radius: 6.4px; border: 2px solid #32443A; text-align: center; padding: 6px; text-shadow: 0 0 6px red; }  .regular-button { background: #795548; color: white; font-family: monospace, sans-serif; font-weight: bold; text-align: center; width: 47.36px; height: 47.36px; border-radius: 5.12px; font-size: 23px; transition: background 0.1s ease-out; box-shadow: 0 3px 7px rgba(0, 0, 0, 0.6); }   .regular-button:active { background: #4e342e; }  .clr-button { background: #9E0E17; font-size: 18px; }  .clr-button:active { background: #DE0814; }  .go-button { background: #43862E; font-size: 18px; }  .go-button:active { background: #4EC527; } </style>  <div class="codelock-area"> <table class="keypad" contenteditable = "false"> <tbody> <tr> <td colspan="3" class="keypad-readout" id="keypad-readout-' + id + '">&zwnj;</th> </tr> <tr> <td class="regular-button" onclick="press(7,' + password + ',' + numLength + ',' + id + ');">7</td> <td class="regular-button" onclick="press(8,' + password + ',' + numLength + ',' + id + ');">8</td> <td class="regular-button" onclick="press(9,' + password + ',' + numLength + ',' + id + ');">9</td> </tr> <tr> <td class="regular-button" onclick="press(4,' + password + ',' + numLength + ',' + id + ');">4</td> <td class="regular-button" onclick="press(5,' + password + ',' + numLength + ',' + id + ');">5</td> <td class="regular-button" onclick="press(6,' + password + ',' + numLength + ',' + id + ');">6</td> </tr> <tr> <td class="regular-button" onclick="press(1,' + password + ',' + numLength + ',' + id + ');">1</td> <td class="regular-button" onclick="press(2,' + password + ',' + numLength + ',' + id + ');">2</td> <td class="regular-button" onclick="press(3,' + password + ',' + numLength + ',' + id + ');">3</td> </tr> <tr> <td class="regular-button clr-button" onclick="press(100,' + password + ',' + numLength + ',' + id + ');">清空</td> <td class="regular-button" onclick="press(0,' + password + ',' + numLength + ',' + id + ');">0</td> <td class="regular-button go-button" onclick="press(200,\'' + password + '\',' + numLength + ',' + id + ');">验证</td> </tr> </tbody> </table> <div class="codelock-info" id="codelock-info-' + id + '"> <div> 密码锁隐藏区域(修改为你想要的内容) </div> <br></div> </div><br>')
}

// 插入可切换Tab区域
function insertSwitchTabArea(row) {
    RE.prepareInsert();
    var rowNum = Number(row)
    var finalHTML = '<br><div class="tabs tabs-style-linebox">'

    // 插入头部区域
    finalHTML += '<nav><ul>'
    for (var i = 0; i < rowNum; i++) {
        if (i == 0) {
            finalHTML += '<li class="tab-current" id="tabview-1"><a href="javascript:void(0);" onclick="switchMindTab(this)">选项卡1</a></li>'
        } else {
            finalHTML += '<li id="tabview-' + (i + 1) + '"><a href="javascript:void(0);" onclick="switchMindTab(this)">选项卡' + (i + 1) + '</a></li>'
        }
    }
    finalHTML += '</ul></nav>'

    // 插入内容区域
    finalHTML += '<div class="tab-content">'
    for (var i = 0; i < rowNum; i++) {
        if (i == 0) {
            finalHTML += '<div class="content-current" id="tabview-1"><div>修改为你需要的内容1</div></div>'
        } else {
            finalHTML += '<div class="content-hide" id="tabview-' + (i + 1) + '"><div>修改为你需要的内容' + (i + 1) + '<br></div></div>'
        }
    }
    finalHTML += '</div><br>'
    RE.insertHTML(finalHTML)
}

// 将数字格式化为字符串，并且保留前置0（不一定正确）
function formatNum(num, length) {
    for (var len = (num + "").length; len < length; len = num.length) {
        num = "0" + num;
    }
    return num;
}

// 密码锁需要的JS
var magicNumber = [, , , , , ,].length;
var emptyReadout = "password";

var maxInputLength = magicNumber;

var inputMap = {}
var busyMap = {}

// 按下密码锁按钮后触发的方法
function press(keyText, realCode, numLength, id) {
    if (inputMap[id] == null) {
        inputMap[id] = []
    }
    if (busyMap[id] == null) {
        busyMap[id] = false
    }

    if (busyMap[id]) {
        return;
    }
    keyText = String(keyText)

    var readout = document.getElementById("keypad-readout-" + id);

    if (keyText === "200") {
        busyMap[id] = true

        if (inputMap[id].join("") == String(realCode)) {
            setTimeout(function () {
                readout.textContent = "密码正确";
            }, 0);
            setTimeout(function () {
                document.getElementById("codelock-info-" + id).style.display = "initial";
            }, 1000);
            setTimeout(function () {
                readout.textContent = emptyReadout;
            }, 2000);
        } else {
            setTimeout(function () {
                readout.textContent = "密码错误";
            }, 0);
            setTimeout(function () {
                readout.textContent = emptyReadout;
            }, 1900);
            setTimeout(function () {
                inputMap[id] = []
                readout.textContent = emptyReadout;
                busyMap[id] = false
            }, 2000);
        }
    }

    else if (keyText === "100") {
        inputMap[id] = []
    }

    else {
        if (inputMap[id].length >= maxInputLength) {
            return;
        }
        inputMap[id].push(keyText);
    }

    readout.textContent = inputMap[id].length === 0 ? emptyReadout : inputMap[id].map(function () {
        return "-";
    }).join("");
}

/**
 * 切换Tab区域
 */
function switchMindTab(obj) {
    var tabRootDiv = obj.parentNode.parentNode.parentNode.parentNode;
    // tabs elems
    var tabs = tabRootDiv.querySelectorAll('nav > ul > li');
    // content items
    var items = tabRootDiv.querySelectorAll('.tab-content > div[id^="tabview"]');

    var currentId = obj.parentNode.id;

    // 处理Tab
    for (var i = 0; i < tabs.length; i++) {
        var tempTab = tabs[i];
        if (tempTab.id == currentId) {
            tempTab.className = 'tab-current';
        } else {
            tempTab.className = '';
        }
    }

    // 处理内容
    for (var i = 0; i < items.length; i++) {
        var tempItem = items[i];
        if (tempItem.id == currentId) {
            tempItem.className = 'content-current';
        } else {
            tempItem.className = 'content-hide';
        }
    }
}

// 插入卡片区域
function insertCardArea() {
    RE.prepareInsert();
    RE.insertHTML('<br><div class="mind-card"><div>&zwnj;</div></div><br>')
}

// 插入信纸格式区域
function insertLetterPaperArea() {
    RE.prepareInsert();
    RE.insertHTML('<br><div class="mind-letter-paper"><div>&zwnj;</div></div><br>')
}

// 插入纯色背景区域
function insertColorBackgroundArea(color) {
    RE.prepareInsert();
    RE.insertHTML('<br><div class="mind-color-area" style="background-color:' + color + '"><div>&zwnj;</div></div><br>')
}

// 插入双端对齐区域
function insertJustifyAlignArea() {
    RE.prepareInsert();
    RE.insertHTML('<div style="text-align:justify;"><div>&zwnj;</div></div>')
}

// 为普通文本或超链接设置/取消下划线
function formatUnderline() {
    // 存在瑕疵，只能去除下划线，无法恢复下划线，且选中的链接必须包含其他元素
    var selectionObj = null, rangeObj = null, selectedHtml = "";
    if (window.getSelection) {
        selectionObj = window.getSelection();
        rangeObj = selectionObj.getRangeAt(0);
        var docFragment = rangeObj.cloneContents();
        var tempDiv = document.createElement("div");
        tempDiv.appendChild(docFragment);
        selectedHtml = tempDiv.innerHTML;
    } else if (document.selection) {
        selectionObj = document.selection;
        rangeObj = selectionObj.createRange();
        selectedHtml = rangeObj.htmlText;
    }
    var tempNode = document.createElement("div");
    tempNode.innerHTML = selectedHtml;
    if (tempNode.getElementsByTagName("a").length > 0) {
        var aNodes = tempNode.getElementsByTagName("a")
        for (var i = 0; i < aNodes.length; i++) {
            if (aNodes[i].style.textDecoration == "none") {
                aNodes[i].style.textDecoration = "underline"
            } else if (aNodes[i].style.textDecoration == "underline" || !aNodes[i].style.textDecoration) {
                aNodes[i].style.textDecoration = "none"
            }
        }
        document.execCommand('insertHTML', false, tempNode.innerHTML);
    }
    document.execCommand('underline', false, null); // 按照常规操作来一下

}