/**
 * 编码/解码工具类
 */
var HtmlUtil = {
    /*1.用浏览器内部转换器实现html编码（转义）*/
    htmlEncode:function (html){
        //1.首先动态创建一个容器标签元素，如DIV
        var temp = document.createElement ("div");
        //2.然后将要转换的字符串设置为这个元素的innerText或者textContent
        (temp.textContent != undefined ) ? (temp.textContent = html) : (temp.innerText = html);
        //3.最后返回这个元素的innerHTML，即得到经过HTML编码转换的字符串了
        var output = temp.innerHTML;
        temp = null;
        return output;
    },
    /*2.用浏览器内部转换器实现html解码（反转义）*/
    htmlDecode:function (text){
        //1.首先动态创建一个容器标签元素，如DIV
        var temp = document.createElement("div");
        //2.然后将要转换的字符串设置为这个元素的innerHTML(ie，火狐，google都支持)
        temp.innerHTML = text;
        //3.最后返回这个元素的innerText或者textContent，即得到经过HTML解码的字符串了。
        var output = temp.innerText || temp.textContent;
        temp = null;
        return output;
    },
    /*3.用正则表达式实现html编码（转义）*/
    htmlEncodeByRegExp:function (str){  
         var temp = "";
         if(str.length == 0) return "";
         temp = str.replace(/&/g,"&amp;");
         temp = temp.replace(/</g,"&lt;");
         temp = temp.replace(/>/g,"&gt;");
         temp = temp.replace(/\s/g,"&nbsp;");
         temp = temp.replace(/\'/g,"&#39;");
         temp = temp.replace(/\"/g,"&quot;");
         return temp;
    },
    /*4.用正则表达式实现html解码（反转义）*/
    htmlDecodeByRegExp:function (str){  
         var temp = "";
         if(str.length == 0) return "";
         temp = str.replace(/&amp;/g,"&");
         temp = temp.replace(/&lt;/g,"<");
         temp = temp.replace(/&gt;/g,">");
         temp = temp.replace(/&nbsp;/g," ");
         temp = temp.replace(/&#39;/g,"\'");
         temp = temp.replace(/&quot;/g,"\"");
         return temp;  
    },
    /*5.用正则表达式实现html编码（转义）（另一种写法）*/
    html2Escape:function(sHtml) {
         return sHtml.replace(/[<>&"]/g,function(c){return {'<':'&lt;','>':'&gt;','&':'&amp;','"':'&quot;'}[c];});
    },
    /*6.用正则表达式实现html解码（反转义）（另一种写法）*/
    escape2Html:function (str) {
         var arrEntities={'lt':'<','gt':'>','nbsp':' ','amp':'&','quot':'"'};
         return str.replace(/&(lt|gt|nbsp|amp|quot);/ig,function(all,t){return arrEntities[t];});
    }
};

// 插入节点区域
function insertNode(nodeName) {
    var html = '<node name="' + nodeName + '">\n\n</node>';
    RE.prepareInsert();
    insertGameContent(html, "node");
}

// 插入文本节点
function insertText() {
    var html = '<text>内容</text>';
    RE.prepareInsert();
    insertGameContent(html, "text");
}

// 插入图像节点
function insertImage(url) {
    var html = '<image url="' + url + '">\n描述（可选）\n</image>';
    RE.prepareInsert();
    insertGameContent(html, "image");
}

// 插入按钮组
function insertBtnGroup() {
    var html = '<btngroup>\n\n</btngroup>';
    RE.prepareInsert();
    insertGameContent(html, "btnGroup");
}

// 插入按钮
function insertButton(title) {
    var html = '<btn value="' + title + '">\n\n</btn>';
    RE.prepareInsert();
    insertGameContent(html, "button");
}

// 插入指令区域
function insertCommand() {
    var html = '<cmd>\n\n</cmd>';
    RE.prepareInsert();
    insertGameContent(html, "command");
}

// 插入条件判断区域
function insertIf() {
    var html = '<if>\n\n</if>';
    RE.prepareInsert();
    insertGameContent(html, "if");
}

// 插入结束指令
function insertEndCommand() {
    var html = 'end()';
    RE.prepareInsert();
    insertGameContent(html, "endCommand");
}

// 插入跳转指令
function insertGotoCommand(nodename) {
    var html = "goto(\""  + nodename +"\")";
    RE.prepareInsert();
    insertGameContent(html, "gotoCommand");
}

// 插入存档指令
function insertSaveCommand(savename) {
    var html = "save(\""  + savename +"\")";
    RE.prepareInsert();
    insertGameContent(html, "saveCommand");
}

// 插入随机变量指令
function insertRandomCommand(name, minValue, maxValue) {
    var html = 'random(' + name + ',' + minValue + ',' + maxValue + ')';
    RE.prepareInsert();
    insertGameContent(html, "randomCommand");
}

// 插入设置变量区域
function insertAttributeCommand(name, valueStr) {
    var html = 'set(' + name + ',' + valueStr + ')';
    RE.prepareInsert();
    insertGameContent(html, "attributeCommand");
}

// 插入设置文本区域
function insertSetTextCommand(name, text) {
    var html = 'set(' + name + ',"' + text + '")';
    RE.prepareInsert();
    insertGameContent(html, "setTextCommand");
}

// 插入从App获取的内容 TODO:集中替换各个标签，确保着色正常
function insertNativeContent(content) {
    if (!content) {
        return
    }
    RE.prepareInsert();
    insertGameContent(content, "appContent")
}

/**
 * 在当前位置插入纯文本内容
 * prepareInsert是为了支持撤销功能
 */
function insertGameContent(content, tag) {
    if (!tag) {
        tag = "default";
    }
    var html = HtmlUtil.htmlEncode(content);
    RE.insertHTML("<pre><code class='" + tag + "'>" + html + "</code></pre>")
}

/**
 * 插入示例代码1
 */
function insertExample1() {
    var content = '<node name="节点1">\n\n<text>节点是基础单位，比如这里就是节点1</text>\n\n<image url="https://files.illund.com/2019/10/31/55bd08df40237f7880f42433486b3d62.jpg">\n单个节点下面可以插入多个文本或图像，比如这就是一张图\n</image>\n\n</node>\n\n\n<node name="node2">\n\n<text>这里是第二个节点区域，演示按钮的使用</text>\n<text>首先需要插入一个按钮组，然后在按钮组里可以插入多个按钮</text>\n\n<btngroup>\n<btn value="按钮1"></btn>\n<btn value="按钮2"></btn>\n</btngroup>\n\n<text>按钮下可以放置多个指令，大家可以参考教程尝试一下</text>\n\n<text>后续内容待更新……</text>\n\n</node>'
    insertNativeContent(content)
}

/**
 * 插入示例代码2
 */
function insertExample2() {
    var content = '<node name="节点1">\n\n<text color="#ffffff" size="14" style="s;b">\n这是我的内容区域\n\n这是第二行\n\n演示了文本风格的使用（删除线、斜体）\n</text>\n\n<cmd>\nset(#a,1)\nset(#a,-1)\n\ngoto("node123")\n\nrandom(#a,10,30)\n\nsave("保存节点")\n</cmd>\n\n<if>\n#A > 8 => goto("node222")\n#B< 99 => random(#a,10,30) | save("临时节点")\n</if>\n\n<btngroup>\n<btn value="按钮1">\n<if>\n#B > 80 => save("测试保存")\n</if>\n</btn>\n\n<btn value="按钮2">\ngoto("节点5")\n</btn>\n\n<btn value="按钮3">\ngoto("节点3")\n</btn>\n\n<btn value="按钮4超过一定的字数">\ngoto("节点3")\n</btn>\n</btngroup>\n\n<text>\n这是我的内容区域2\n</text>\n\n<cmd>\ngoto("节点3")\n</cmd>\n</node>\n\n<node name="节点2">\n\n<text bgcolor="#ffff00" color="#ff0000" size="14" style="s;b">\n节点2的内容区域\n</text>\n\n<image bgcolor="#ffff00" color="#ff0000" size="14" url="https://timgsa.baidu.com/timg?image&quality=80&size=b9999_10000&sec=1570173602066&di=fedc979462d8b8eb7ae96f70c49f4e2a&imgtype=0&src=http%3A%2F%2Fi0.hdslb.com%2Fbfs%2Farticle%2Fe90c7218da3e8da0519efc38aa4c488837fa88a1.jpg">\n节点2的测试图片区域\n</image>\n\n</node>\n\n<node name="节点3">\n\n<text color="#111111" bgcolor="#00ff00" size="14" style="s;b">\n节点3的内容区域\n</text>\n\n<image size="18" url="https://timgsa.baidu.com/timg?image&quality=80&size=b9999_10000&sec=1570173602066&di=fedc979462d8b8eb7ae96f70c49f4e2a&imgtype=0&src=http%3A%2F%2Fi0.hdslb.com%2Fbfs%2Farticle%2Fe90c7218da3e8da0519efc38aa4c488837fa88a1.jpg">\n节点3的测试图片区域\n</image>\n\n<cmd>\ngoto("节点2")\n</cmd>\n\n</node>'
    insertNativeContent(content)
}

/**
 * 插入示例代码3
 */
function insertExample3() {
    var content = '<node name="我的节点1">\n\n<image url="https://files.illund.com/2019/10/14/6e66d64e40a6622f80d77e557229631a.gif">\n在这里输入图片下方的文字（可选）</image>\n\n<cmd>\nset(#A,0)\n</cmd>\n\n<text>\n接下来我们来玩操作生命值的游戏\n</text>\n\n</node>\n\n<node name="操作生命值">\n\n<btngroup>\n<btn value="生命值恢复100">\nset(#A,100)\n</btn>\n\n<btn value="生命值-20">\nset(#A,-20)\n</btn>\n\n<btn value="生命值随机">\nrandom(#A,0,200)\n</btn>\n\n<btn value="保存">\nsave("我的存档")\n</btn>\n\n<btn value="HP #A">\n</btn>\n</btngroup>\n\n<cmd>\ngoto("生命值")\n</cmd>\n\n</node>\n\n<node name="生命值">\n\n<text>\n你的当前生命值为：#A\n</text>\n\n<cmd>\ngoto("操作生命值")\n</cmd>\n\n</node>'
    insertNativeContent(content)
}


// /**
//  * 在当前位置插入纯文本内容
//  * prepareInsert是为了支持撤销功能
//  */
// function insertGameContent(content) {
//     RE.prepareInsert();
//     document.execCommand('insertText', false, content);
// }